# $Id: gsasmenu.tcl,v 1.9 1999/09/09 19:06:45 toby Exp toby $
# $Revision: 1.9 $ $Date: 1999/09/09 19:06:45 $
# menu information for GSAS programs

# menu items to be created (N.B. File, Options & Help already exist)
set expgui(menunames) {Powder Xtal Graphs Results Calc Import/Export}

# contents of each menu
array set expgui_menulist {
    file {
	convert
	dlst
    }
    option {
	{liveplot_options}
    }
    powder {
	expedt
	powpref
	genles
	powplot
	rawplot
	fitspec
	tofnorm
    }
    xtal {
	expedt
	genles
	scabs
	scmerge
	sxtldata
    }
    graphs {
	forplot
	polfplot
	powplot
	ortep
	rawplot
	fourier
	forsrh
	liveplot
	vrstplot
	widplt 
    }
    calc {
	cllchg
	fprime
	hklgen
	rducll
	spcgroup
	unimol
    }
    import/export {
	exp2shelx
	exp2xtl
	gsas2cif
	hklsort
	pubtable
	convert
	cad4rd
	dbwscnv
	x17bcnv
	p3r3data
	sxtldata
    }
    results {
	bijcalc
	disagl
	reflist
	geometry
	hstdmp
	istats
	rcalc
	composition
	lstview
	last_r
    }
}

array set expgui_cmdlist {

    Save {- {
	Saves modifications to the current experiment file to disk}
    }

    {Save As} {- {
	Saves modifications to the current experiment file to disk
	under a new file name }
    }

    {Reread .EXP file} {- {
	Reread the last saved version of the experiment file from disk.

	This causes any unsaved changes to be lost.}
    }

    {Sort atoms by}  {- {
	Determines the order that atoms are displayed on the "Phase" page
	Atoms may be displayed sorted by atom number, atom type, 
	or by x, y or z}
    }

    {Sort histograms by}  {- {
	Determines the order that histograms are displayed on the 
	Histogram, Scaling and Profile pages

	Histograms may be sorted by histogram number, histogram type, 
	original bank number, or diffraction angle/wavelength}
    }

    {Multiple hist. selection}  {- {
	When this mode is off, it is possible to modify parameters
	and refinement flags for only a single histogram. For other settings,
	it is possible to modify parameters and flags for groups of 
	histograms (see help for Mouse actions). 

	It does not make sense, however, to globally modify
	instrument-related parameters and flags for different histogram types.
	So global actions can be limited to a single class of histogram types
	(e.g. TOF, CW Neutron,...), which allows these parameters to be set
	for groups of similar histograms. Thus, if this mode is set to "All"
	the Histogram and Profile pages are disabled.}
    }

    {Mouse actions}  {- {
	A range of atoms or (in multiple selection mode) histograms may be 
	selected by dragging (holding down) the left mouse button. It is also 
	possible to select a range by using the Shift key with the 
	left mouse button. To select or deselect individual entries, use the 
	Control key with the left mouse button. The right mouse button selects
	all entries.}
   }

    expnam {readnewexp {
    	Select an existing or new GSAS experiment to be used}
    }

    {archive EXP} {- {
	Toggles archiving of .EXP files. When on, files are 
	saved prior to each save or run of expedt in a file named 

	<expnam>.EXP.xxx.gz where xxx = 000, 001 (UNIX) 

	or in a file named <expnam>.ZIP or  <expnam>.xxx (Windows) }
    }

    showhelp    {showhelp {
	Show the help information for commands and actions}
    }

    powpref	{{runGSASwEXP $cmd} {
	Powder data preparation}   
    }

    bijcalc	{{runGSASwEXP $cmd} {
	Thermal parameter analysis}  
    }

    powplot	{{runGSASwEXP $cmd} {
	Display powder patterns}
    }

    cllchg	{{runGSASwEXP $cmd} {
	Transform unit cell}
    }

    expedt	{{runGSASwEXP $cmd} {
	Run GSAS experiment editor}
    }

    genles	{{runGSASwEXP $cmd} {
	Run General Least Squares program}
    }

    disagl	{{runGSASwEXP $cmd} {
	Distance/angle calculations}
    }

    forplot	{{runGSASwEXP $cmd} {
	Display Fourier maps (set Fourier options in EXPEDT 
	and then compute with FOURIER)}
    }

    hstdmp	{{runGSASwEXP $cmd} {
	List powder histogram data}
    }

    cad4rd	{{runGSASwEXP $cmd} {
	Prepare CAD4 single crystal data}
    }

    fourier	{{runGSASwEXP $cmd} {
	Generate Fourier map}
    }

    geometry	{{runGSASwEXP $cmd} {
	Molecular geometry calculations}
    }

    ortep	{{runGSASwEXP $cmd} {
	Draw crystal structure}
    }

    rawplot	{{runGSASprog $cmd} {
	Plot powder data}
    }


    p3r3data	{{runGSASwEXP $cmd} {
	Prepare Siemens/Brucker P3R3 single crystal data}
    }

    forsrh	{{runGSASwEXP $cmd} {
	Search Fourier map for peaks}
    }

    hklsort	{{runGSASwEXP $cmd} {
	Prepare HKL tables}
    }

    polfplot	{{runGSASwEXP $cmd} {
	Display polefigures}
    }

    rducll	{{runGSASprog $cmd} {
	Unit cell reduction}
    }

    sxtldata	{{runGSASwEXP $cmd} {
	Prepare generic single crystal data}
    }

    scabs	{{runGSASwEXP $cmd} {
	Single crystal absorption}
    }

    istats	{{runGSASwEXP $cmd} {
	HKL Intensity statistics}
    }

    reflist	{{runGSASwEXP $cmd} {
	List reflection data}
    }

    scmerge	{{runGSASwEXP $cmd} {
	Sort and merge single crystal data}
    }

    pubtable	{{runGSASwEXP $cmd} {
	Prepare atom parameter tables}
    }

    spcgroup	{{runGSASprog $cmd} {
	Space group symbol interpreter}
    }

    rcalc	{{runGSASwEXP $cmd} {
	Compute reflection resuduals}
    }

    unimol	{{runGSASwEXP $cmd} {
	Unique molecule assembler}
    }

    gsas2cif	{{runGSASwEXP $cmd} {
	Prepare IUCr crystallographic information (CIF) file}
    }

    vrstplot    {{runGSASwEXP $cmd} {
	Create a "virtual reality" (.wrl) plot file}
    }

    fitspec	{{runGSASprog $cmd} {
	Fit a TOF vanadium scattering spectrum}
    }

    tofnorm	{{runGSASprog $cmd} {
	Normalize a TOF spectrum}
    }

    fprime	{{runGSASprog $cmd} {
	Compute f, f', f'' and mu/rho for an element for a range of x-ray wavelengths}
    }

    dbwscnv	{{runGSASprog $cmd} {
	Convert a powder diffraction data file from DBWS format}
    }

    x17bcnv	{{runGSASprog $cmd} {
	Convert an energy dispersive diffractogram data file from NSLS X17b}
    }

    composition	{{composition} {
	Compute the unit cell and asymmetric unit contents for each phase
	taking occupancies and site multiplicities into account}
    }

    exp2xtl	{{exp2xtl} {
	Save coordinates for a phase in an MSI xtl format file}
    }

    liveplot	{{liveplot} {
	Create a plot of powder data (automatically updated) }
    }

    {liveplot_options} {liveplotopt {
	Used to set options for liveplot, for example, the 
	histogram to be plotted}
    }
    
    convert	{convfile {
	Convert a standard ASCII file to the direct access format used by GSAS (and for UNIX, the reverse)}
    }

    lstview     {lstview {
	Create a box with scrollbars containing the current .LST file}
    }

    widplt	{widplt {
	Displays the FWHM as a function of Q, 2Theta,... for UVWXY values input or read from an EXP file}
    }

    "Override backspace" {- {
	This option is available in UNIX only, as there are different
	ways that backspace can be implemented. When option is set 
	as "On," the backspace key is overridden to send a "delete" 
	character. If backspace does not work in a program such as 
	EXPEDT, change try the other setting for this option.}	
    }
    SaveOptions {- {
	Save the current values for "Override backspace", 
	"Sort atoms by", "Sort histograms by" and "archive EXP"
	in ~/.gsas_config (c:\.gsas_config)}
    }
    exit	{- {
	Exit EXPGUI}
    }
}

# not implemented (yet)
#    newest	{}
#composition	{{} {}}
#exp2shelx	{{} {}}
#exp2xtl	{{} {}}
#dlst	{{delconf .LST} {}}

set expgui(buttonlist) {
	expnam
	expedt
	genles
	powpref
	powplot
	lstview
	liveplot
}	
